#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
# Author: Miguel Ávarez <malvarez@redborder.com>
#######################################################################

#
# Base redBorder module
#
module Redborder
  #
  # Module to download TI data from remote FTP server and inject
  # into memcached for later enrichment in logstash pipelines
  #
  module ThreatIntel
    class Client < Redborder::TIConnections::Client
      #
      # Initialize the ThreatIntel client with FTP connection and Dalli caching client.
      #
      # == Parameters:
      # protocol::
      #   A string representing the protocol to use for FTP connection (ftp, sftp, or ftps).
      # host::
      #   A string representing the host of the FTP server.
      # user::
      #   A string representing the username for authentication.
      # password::
      #   A string representing the password for authentication.
      # dalli_options::
      #   A hash containing the Dalli client options. Must include :server and :options.
      #
      # == Returns:
      # A new instance of Redborder::ThreatIntel::Client.
      #
      def initialize(protocol:, host:, port:, user:, password:, dalli_options: {})
        super(protocol: protocol, host: host, port: port, user: user, password: password)
        @dalli_client = Dalli::Client.new(dalli_options[:servers], dalli_options[:options])
      end

      #
      # Download a file from the remote FTP server and inject its content into a Dalli cache.
      #
      # == Parameters:
      # remote_path::
      #   A string representing the remote file path to download.
      # local_path::
      #   A string representing the local file path to store the downloaded file.
      #
      # == Returns:
      # None.
      #
      # == Raises:
      # Net::FTPError::
      #   If there is an FTP error during the download process.
      # IOError::
      #   If there is an I/O error during the download process.
      # ArgumentError::
      #   If there is an argument error during the download and injection process.
      #
      def download_and_inject(remote_path:, local_path:)
        download_file(remote_path, local_path)
        inject(local_path)
      rescue Net::FTPError => e
        Rails.logger.error "FTP error in download_and_inject: #{e.message}"
        raise
      rescue IOError => e
        Rails.logger.error "I/O error in download_and_inject: #{e.message}"
        raise
      rescue ArgumentError => e
        Rails.logger.error "Argument error in download_and_inject: #{e.message}"
        raise
      end

      private

      #
      # Process the file line by line and inject each line as a key into the Dalli cache.
      #
      # == Parameters:
      # file_path::
      #   A string representing the path to the local file to inject into Dalli cache.
      #
      # == Returns:
      # None.
      #
      # == Raises:
      # Dalli::DalliError::
      #   If there is an error while setting keys into Dalli cache.
      # IOError::
      #   If there is an I/O error during the file processing.
      # ArgumentError::
      #   If there is an argument error during the injection process.
      #
      def inject(file_path)
        File.foreach(file_path) do |line|
          begin
            key = line.strip
            next if key.blank?
      
            @dalli_client.set(key, 1)
          rescue Dalli::DalliError => e
            Rails.logger.error "Dalli error while injecting key '#{key}': #{e.message}"
            raise
          end
        end
      rescue IOError => e
        Rails.logger.error "I/O error during file processing: #{e.message}"
        raise
      rescue ArgumentError => e
        Rails.logger.error "Argument error in inject: #{e.message}"
        raise
      end      
    end
  end
end